use strict;
use warnings;
use lib 't/lib';
use Test::More;
use My::Schema;

my $schema = My::Schema->connect('dbi:SQLite:dbname=:memory:');

$schema->deploy;

my $people = $schema->resultset('Person');

my $bob = $people->create({
    name => 'Bob Testuser',
});

$bob->create_related(assigned_tasks => {
    summary => 'Task A',
    urgency => 10,
});
$bob->create_related(assigned_tasks => {
    summary => 'Task B',
    urgency => 20,
});
$bob->create_related(assigned_tasks => {
    summary => 'Task C',
    urgency => 30,
});

subtest 'has_many' => sub {

    my $join_with_min = sub {
        return shift->with_parameterized_join(
            urgent_assigned_tasks => { urgency_threshold => $_[0] },
        );
    };

#    my $join_with_range = sub {
#        return shift->with_parameterized_join(
#            tasks_in_urgency_range => {
#                min => $_[0],
#                max => $_[1],
#            },
#        );
#    };

    my $search = sub {
        return scalar shift->search(
            { 'me.name' => { -like => 'Bob%' } },
            {
                '+select' => [{
                    count => \['urgent_assigned_tasks.id'],
                }],
                '+as' => ['task_count'],
            },
        );
    };

    my $fetch_count = sub {
        return shift->next->get_column('task_count');
    };

    subtest 'simple filter' => sub {
        is $people->$join_with_min(19)->$search->$fetch_count,
            2, 'filter min 19';
        is $people->$join_with_min(29)->$search->$fetch_count,
            1, 'filter min 29';
        is $people->$join_with_min(39)->$search->$fetch_count,
            0, 'filter min 39';
    };

    subtest 'multiple filters' => sub {
        my $rs1 = $people->$join_with_min(19)->$search;
        my $rs2 = $people->$join_with_min(29)->$search;
        is $rs1->$fetch_count, 2, 'first';
        is $rs2->$fetch_count, 1, 'second';
    };

    subtest 'overrides' => sub {
        is $people
            ->$join_with_min(190)
            ->$join_with_min(19)
            ->$search
            ->$fetch_count,
            2, 'overridden parameter';
    };

#    subtest 'multi parameter' => sub {
#        is $people->$join_with_range(10, 30)->$search->$fetch_count,
#            3, 'full range';
#    };
};

done_testing;
